# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

require 'ams_Lib.rb'

module AMS

  module Lib

    module_function

    PATH = File.dirname(__FILE__)
    BASE = File.basename(__FILE__)

    @loaded_files = [] unless defined?(@loaded_files)

    # Retrieves a name to the main file in the library's directory.
    # @return [String]
    def basename
      BASE
    end


    # Retrieves path to the library's directory.
    # @return [String]
    def path
      PATH
    end


    # Requires the specified file in the library's directory.
    # @param [String] file A file name with an extension.
    # @return [Boolean] +true+ if successful, +false+ if not.
    def require(file)
      file = File.join(path, file) unless File.exists?(file)
      return false unless File.exists?(file)
      ext = File.extname(file)
      return false if ext.empty? or @loaded_files.include?(file)
      state = Kernel.require(file)
      @loaded_files.push(file) unless @loaded_files.include?(file)
      state
    end


    # Requires all files in the library's directory.
    # @return [Array<String>] An array of loaded files.
    def require_all
      files = Dir.glob( File.join(path, '*.{rb, rbs, so, dll}') )
      loaded = []
      files.each{ |file|
        next if (File.basename(file) == basename)
        state = self.require(file)
        loaded.push(file) if state
      }
      loaded
    end


    # Requires the specified file in the <i>ams_Lib/libraries/</i> directory.
    # @param [String] file A file name with an extension.
    # @return [Boolean] +true+ if successful, +false+ if not.
    def require_library(file)
      file = File.join(path, 'libraries', file)
      self.require(file)
    end


    # Requires all files in the <i>ams_Lib/libraries/</i> directory.
    # @return [Array<String>] An array of loaded files.
    def require_libraries
      files = Dir.glob( File.join(path, 'libraries', '*.{rb, rbs, so}') )
      loaded = []
      files.each{ |file|
        state = self.require(file)
        loaded.push(file) if state
      }
      loaded
    end


    # Loads the specified file in the library's directory.
    # @param [String] file A file name with _.rb_ or _.rbs_ extension.
    # @return [Boolean] +true+ if successful, +false+ if not.
    def load(file)
      file = File.join(path, file) unless File.exists?(file)
      return false unless File.exists?(file)
      ext = File.extname(file)
      return false if ext.empty?
      state = Kernel.load(file)
      @loaded_files.push(file) unless @loaded_files.include?(file)
      state
    end


    # Loads all files in the library's directory.
    # @return [Array<String>] An array of loaded files.
    def load_all
      files = Dir.glob( File.join(path, '*.{rb, rbs}') )
      loaded = []
      files.each{ |file|
        next if (File.basename(file) == basename)
        state = self.load(file)
        loaded.push(file) if state
      }
      loaded
    end


    # Loads the specified file in the <i>ams_Lib/libraries/</i> directory.
    # @param [String] file A file name with an extension.
    # @return [Boolean] +true+ if successful, +false+ if not.
    def load_library(file)
      file = File.join(path, 'libraries', file)
      self.load(file)
    end


    # Loads all files in the <i>ams_Lib/libraries/</i> directory.
    # @return [Array<String>] An array of loaded files.
    def load_libraries
      files = Dir.glob( File.join(path, 'libraries', '*.{rb, rbs}') )
      loaded = []
      files.each{ |file|
        state = self.load(file)
        loaded.push(file) if state
      }
      loaded
    end


    # Retrieves an array of loaded files.
    # @return [Array<String>] An array of full path files.
    def loaded_files
      @loaded_files.dup
    end


    # Registers the specified file into the +loaded_files+ list.
    # @param [String] file A file name with an extension.
    # @return [Boolean] +true+ if the file is valid, +false+ if not.
    def file_loaded(file)
      file = File.join(path, file) unless File.exists?(file)
      return false unless File.exists?(file)
      ext = File.extname(file)
      return false if ext.empty? or @loaded_files.include?(file)
      @loaded_files.push(file)
      true
    end


    # Determines whether the specified file has been loaded.
    # @param [String] file A file name.
    # @return [Boolean] +true+ if the file is loaded, +false+ if not.
    def file_loaded?(file)
      @loaded_files.include?(file)
    end

  end # module Lib


  # This module is the namespace for all example scripts.
  module Examples; end

  # This module is the namespace for all plugins/extensions.
  module Plugins; end

  # This module is the namespace for all test scripts.
  module Test; end

end # module AMS

AMS::Lib.file_loaded(__FILE__)
