# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

dir = File.dirname(__FILE__)
require File.join(dir, 'core.rb')

class AMS::MultiLineText

  # Creates a new multi-line text.
  #
  # @param [Integer] x X position.
  # @param [Integer] y Y position.
  # @param [String] intro An introduction string to be displayed on top of the
  #   multi-line text.
  def initialize(x = 10, y = 10, intro = "")
    @text = []
    @ent = nil
    @limit = 20
    @lines = 0
    @pos = [x,y]
    @intro = ""
    @hl = '-' * 80
    set_intro intro
  end

  attr_reader :limit, :lines

  private

  def add_text(text = '')
    model = Sketchup.active_model
    view = model.active_view
    w = view.vpwidth
    h = view.vpheight
    hr = @pos[0]/w.to_f
    vr = @pos[1]/h.to_f
    @ent = model.add_note(text, hr, vr)
  end


  def update
    add_text if deleted?
    len = @text.length
    @text[0, (len - @limit)] = nil if len > @limit
    if @intro.empty?
      @ent.text = @text.join("\n")
    else
      @ent.text = "#{@hl}\n\n#{@intro}\n\n#{@hl}\n\n#{@text.join("\n")}"
    end
    nil
  end


  def deleted?
    @ent == nil or @ent.deleted?
  end

  public

  # Assigns a new limit to the number of lines to be displayed.
  # @param [Integer] lim Limit
  # @return [Integer] The new limit
  def limit=(lim)
    @limit = lim.to_i
    @limit = 1 if lim < 1
    @limit = 140 if lim > 140
    update
    @limit
  end


  # Sets a new description text to be displayed on top of the multi-line text.
  # @param [String] text The new introduction string.
  # @return [nil]
  def set_intro(text = "")
    @intro = text.to_s
    update
  end


  # Prints the specified string on the current line of the multi-line text.
  # @param [String] str Text
  # @return [nil]
  def write(str)
    @text.last.push(str.to_s)
    update
  end


  # Prints the specified string on a new line of the multi-line text.
  # @param [String] str Text
  # @return [nil]
  def writeln(str)
    @lines += 1
    @text.push("#{sprintf("[%03i]", @lines)} #{str}")
    update
  end


  # Clears text and its data.
  # @return [nil]
  def clear
    @text.clear
    @lines = 0
    update
    nil
  end


  # Deletes the text entity and clears all data.
  # @return [nil]
  def remove
    @ent.erase! unless deleted?
    @ent = nil
    @text.clear
    @lines = 0
    nil
  end

end # class AMS::MultiLineText

AMS::Lib.file_loaded(__FILE__)
