# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

dir = File.dirname(__FILE__)
require File.join(dir, 'core.rb')

# The ray utility adds more options to the +Sketchup.active_model.raytest+
# method.
module AMS::RayUtil

  module_function

  # Retrieves an array of points intersecting the ray.
  # _T1_ : type 1 checks all model entities.
  # @param [Geom::Point3d, Array] point Ray position.
  # @param [Geom::Vector3d, Array] vector Ray direction.
  # @param [Boolean] nchg Whether to not consider hidden geometry.
  # @return [Array<Geom::Point3d>] An array of points intersecting the ray.
  def deepray_t1(point, vector, nchg = false)
    nchg = (nchg == true)
    ray = [point,vector]
    pts = []
    result = nil
    while true
      result = Sketchup.active_model.raytest(ray, nchg)
      break unless result
      x = result[0]
      pts.push x
      ray[0] = x
    end
    pts
  end


  # Retrieves an array of points intersecting the ray.
  # _T2_ : type 2 checks all the given entities.
  # @param [Array<entity>] ents An array of entities to include.
  # @param [Geom::Point3d, Array] point Ray position.
  # @param [Geom::Vector3d, Array] vector Ray direction.
  # @param [Boolean] nchg Whether to not consider hidden geometry.
  # @return [Array<Geom::Point3d>] An array of points intersecting the ray.
  def deepray_t2(ents, point, vector, nchg = false)
    ents = [ents] unless ents.is_a?(Array)
    nchg = (nchg == true)
    ray = [point,vector]
    pts = []
    result = nil
    while true
      result = Sketchup.active_model.raytest(ray, nchg)
      break unless result
      x = result[0]
      pts.push x if ents.include?(result[1][0])
      ray[0] = x
    end
    pts
  end


  # Retrieves an array of points intersecting the ray.
  # _T3_ : type 3 checks all, but the given entities.
  # @param [Array<entity>] ents An array of entities to ignore.
  # @param [Geom::Point3d, Array] point Ray position.
  # @param [Geom::Vector3d, Array] vector Ray direction.
  # @param [Boolean] nchg Whether to not consider hidden geometry.
  # @return [Array<Geom::Point3d>] An array of points intersecting the ray.
  def deepray_t3(ents, point, vector, nchg = false)
    ents = [ents] unless ents.is_a?(Array)
    nchg = (nchg == true)
    ray = [point,vector]
    pts = []
    result = nil
    while true
      result = Sketchup.active_model.raytest(ray, nchg)
      break unless result
      x = result[0]
      pts.push x unless ents.include?(result[1][0])
      ray[0] = x
    end
    pts
  end


  # Casts a ray through the model and return the first thing that the ray hits.
  # _T1_ : type 1 checks all given entities.
  # @param [Array<entity>] ents An array of entities to include.
  # @param [Geom::Point3d, Array] point Ray position.
  # @param [Geom::Vector3d, Array] vector Ray direction.
  # @param [Boolean] nchg Whether to not consider hidden geometry.
  # @return [Array, nil] A ray result:
  #   http://www.sketchup.com/intl/en/developer/docs/ourdoc/model.php#raytest
  def raytest_t1(ents, point, vector, nchg = false)
    ents = [ents] unless ents.is_a?(Array)
    nchg = (nchg == true)
    ray = [point,vector]
    result = nil
    while true
      result = Sketchup.active_model.raytest(ray, nchg)
      break unless result
      return result if ents.include?(result[1][0])
      ray[0] = result[0]
    end
    nil
  end


  # Casts a ray through the model and return the first thing that the ray hits.
  # _T2_ : type 2 checks all, but the given entities.
  # @param [Array<entity>] ents An array of entities to ignore.
  # @param [Geom::Point3d, Array] point Ray position.
  # @param [Geom::Vector3d, Array] vector Ray direction.
  # @param [Boolean] nchg Whether to not consider hidden geometry.
  # @return [Array, nil] A ray result:
  #   http://www.sketchup.com/intl/en/developer/docs/ourdoc/model.php#raytest
  def raytest_t2(ents, point, vector, nchg = false)
    ents = [ents] unless ents.is_a?(Array)
    nchg = (nchg == true)
    ray = [point,vector]
    result = nil
    while true
      result = Sketchup.active_model.raytest(ray, nchg)
      break unless result
      return result unless ents.include?(result[1][0])
      ray[0] = result[0]
    end
    nil
  end

end # module AMS::RayUtil

AMS::Lib.file_loaded(__FILE__)
