# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

Sketchup.require 'ams_Lib/sketchup_window.rb'

# Menubar is a strip with SU commands. It is located below the window's caption.
module AMS::Sketchup::Menubar

  class << self

    private

    def win
      AMS::Windows
    end

    public

    # Retrieves a handle to the main window's menu-bar.
    # @return [Integer] hMenu
    def hmenu
      AMS::Sketchup.menubar
    end

    alias handle hmenu


    # Sets or removes menubar from the main window.
    # @note Menu hot key operations won't work without the menubar present.
    # @param [Boolean] state +true+ : set, +false+ : remove.
    # @return [Boolean] Whether the menubar state was changed.
    def set(state = true)
      state = (state == true)
      hMenu = (state ? hmenu : nil)
      return false if (state == set?)
      show(true) unless state
      win::EndMenu.call()
      win::SetMenu.call(AMS::Sketchup.main_window, hMenu)
      win::DrawMenuBar.call(AMS::Sketchup.main_window)
      true
    end


    # Determines whether the current menubar is set.
    # @return [Boolean] Whether the current menubar is there.
    def set?
      win::GetMenu.call(AMS::Sketchup.main_window) == hmenu
    end


    # Removes menubar for a specified time in seconds.
    # @param [Float] time
    # @return [nil]
    def remove_for_time(time = 60)
      time = time.abs
      maintain false
      t = Object::UI.start_timer(time, false){
        Object::UI.stop_timer t
        maintain true
      }
      nil
    end


    # Shows or hides menubar of the main window. This function sizes the main
    # window that way, so that menubar appears above the screen. Perhaps
    # "hiding" menubar becomes an advantage over removing it because menu hot
    # key opearions won't work without it.
    # @note This can be done when the main window is fullscreen only. This also
    #   requires the menubar to set.
    # @note You'll still be able to access menu by pressing the +Alt+ key.
    # @param [Boolean] state +true+ : show, +false+ : hide.
    # @return [Boolean] +true+ if +successful+ and visibility state changed,
    #   +false+ if not.
    def show(state = true)
      state = (state == true)
      return false unless set?
      return false unless AMS::Sketchup::MainWindow.fullscreen?
      return false if state == visible?
      main = AMS::Sketchup.main_window
      w = win::GetSystemMetrics.call(0) # SM_CXSCREEN
      h = win::GetSystemMetrics.call(1) # SM_CYSCREEN
      mh = win::GetSystemMetrics.call(15) # SM_CYMENU
      if state
        win::SetWindowPos.call(main, nil, 0, 0, w, h, 0x0644)
      else
        win::SetWindowPos.call(main, nil, 0, -mh, w, h+mh, 0x0644)
      end
      true
    end


    # Determines whether menubar of the main window is visible.
    # @return [Boolean] +true+ : visible, +false+ : hidden or removed.
    def visible?
      return false unless set?
      return true unless AMS::Sketchup::MainWindow.fullscreen?
      main = AMS::Sketchup.main_window
      mh = win::GetSystemMetrics.call(15) # SM_CYMENU
      rect = AMS::Window.rect(main)
      rect[1] > -mh
    end

  end # proxy class

end # module AMS::Sketchup::MenuBar

AMS::Lib.file_loaded(__FILE__)
