# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

Sketchup.require 'ams_Lib/sketchup_window.rb'

# This observer is implemented to react to SketchUp window events. As you've
# noticed, a special _swo_ extension is added in front of each observer. This is
# to make it distinguishable and compatible among other callback methods if they
# are mixed in the same class. FYI: _swo_ stands for <i>Sketchup Window Observer</i>.
# @example
#   class MySUObserver
#
#     def mw_onSwitchFullscreen(state)
#       if state
#         puts 'Main window has been set fullscreen.'
#       else
#         puts 'Main window has been set to original placement.'
#       end
#     end
#
#     def mw_onActivate
#       puts 'Main window has been set foreground.'
#     end
#
#     def vp_onRepaint
#       puts 'Viewport has been redrawn.'
#     end
#
#   end
#
#   # Attach the observer
#   AMS::Sketchup.add_observer(MySUObserver.new)
module AMS::Sketchup::Observer

  # Called when the observer is added.
  def swo_activate
  end


  # Called when the observer is removed. The observer is removed either because
  # of an error in one of the callback methods, or because user called
  # +AMS::Sketchup.remove_observer(obj)+ method.
  def swo_deactivate
  end


  # Called when main window is assigned new title text.
  # @param [String] text The new title text.
  def swo_mw_onCaptionChanged(text)
  end


  # Called when main window is set fullscreen or set to original placement.
  # @param [Boolean] state +true+ : set fullscreen, +false+ : set to original
  #   placement.
  def swo_mw_onSwitchFullscreen(state)
  end


  # Called when main window is maximized.
  def swo_mw_onMaximize
  end


  # Called when main window is minimized.
  def swo_mw_onMinimize
  end


  # Called when main window is set to default placement.
  def swo_mw_onRestore
  end


  # Called when main window is set foreground.
  def swo_mw_onActivate
  end


  # Called when main window looses focus.
  def swo_mw_onDeactivate
  end


  # Called after main window enters the moving or sizing modal loop.
  def swo_mw_onEnterSizeMove
  end


  # Called after main window exits the moving or sizing modal loop.
  def swo_mw_onExitSizeMove
  end


  # Called when main window is moved or sized.
  # @param [Integer] x1 X position of the upper-left corner of the window.
  # @param [Integer] y1 Y position of the upper-left corner of the window.
  # @param [Integer] x2 X position of the lower-right corner of the window.
  # @param [Integer] y2 Y position of the lower-right corner of the window.
  def swo_mw_onSizeMove(x1,y1, x2,y2)
  end


  # Called when key is pressed.
  # @param [String] key Virtual key name
  # @param [Integer] val Virual key value
  def swo_mw_onKeyDown(key, val)
  end


  # Called when key is held down.
  # @param [String] key Virtual key name
  # @param [Integer] val Virual key value
  def swo_mw_onKeyExtended(key, val)
  end


  # Called when key is released.
  # @param [String] key Virtual key name
  # @param [Integer] val Virual key value
  def swo_mw_onKeyUp(key, val)
  end


  # Called when menubar is set or removed.
  def swo_mb_onSetRemove(state)
  end


  # Called when menubar is shown or hidden.
  # @note: The visibility state of menubar can be changed while main window is
  # set fullscreen only.
  # @param [Boolean] state +true+ : set visible, +false+ : set hidden.
  def swo_mb_onVisibilityChanged(state)
  end


  # Called when viewport border is set or removed.
  # @param [Boolean] state +true+ : set, +false+ : removed.
  def swo_vp_onSetRemoveClientEdge(state)
  end


  # Called when viewport is sized.
  # @param [Integer] w The new width of the window.
  # @param [Integer] h The new height of the window.
  def swo_vp_onSize(w,h)
  end


  # Called when mouse enters the viewport client area.
  def swo_vp_onMouseEnter
  end


  # Called when mouse exits the viewport client area.
  def swo_vp_onMouseLeave
  end


  # Called when the viewport window is repainted. This callback is called often,
  # so be careful at what you put here.
  def swo_vp_onRepaint
  end


  # Called when a toolbar container is shown or hidden.
  # @param [Integer] bar +1+ : top, +2+ : bottom, +3+ : left, +4+ : right.
  # @param [Boolean] state +true+ : set visible, +false+ : set hidden.
  def swo_tbc_onVisibilityChanged(bar, state)
  end


  # Called when scenesbar is shown or hidden.
  # @param [Boolean] state +true+ : set visible, +false+ : set hidden.
  def swo_scb_onVisibilityChanged(state)
  end


  # Called when statusbar is shown or hidden.
  # @param [Boolean] state +true+ : set visible, +false+ : set hidden.
  def swo_stb_onVisibilityChanged(state)
  end

end

AMS::Lib.file_loaded(__FILE__)
