# Input Procedure is the interface that you implement to create an interactive
# tool or game in SketchUp. Unlike the SketchUp tool this lets you decide
# whether the event should interact with SketchUp. For instance, returning +1+
# in an +onKeyDown+ will prevent the message from interfering with SketchUp
# shortcuts. As you noticed a special _hk_ extension is added to the beginning
# of each callback function. This is to ensure compatibility and stability when
# you use the same class for input procedure and SketchUp tool.
# See <i>examples/ams_InputProc.rb</i> for detailed implementation.
# @example
#   class MyTool
#
#     def hk_onKeyDown(key, val)
#       if (key == 'escape')
#         AMS::InputProc.deselect_tool self
#       else
#         puts "onKeyDown #{key}"
#       end
#       # Prevent this key from interacting with SketchUp.
#       return 1
#     end
#
#   end
#
#   # Select the tool
#   AMS::InputProc.select_tool(MyTool.new, true, false, false)
module AMS::InputProc::Tool

  # Called when the tool is selected.
  def hk_activate
  end


  # Called when the tool is deselected. A tool is deselected either because of
  # an error in one of the callback methods, or because a different tool was
  # selected.
  def hk_deactivate
  end


  # Called when key is pressed.
  # @param [String] key Virtual key name
  # @param [Integer] val Virual key value
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onKeyDown(key, val)
  end


  # Called when key is released.
  # @param [String] key Virtual key name
  # @param [Integer] val Virual key value
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onKeyUp(key, val)
  end


  # Called when key is held down.
  # @param [String] key Virtual key name
  # @param [Integer] val Virual key value
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onKeyExtended(key, val)
  end


  # Called whenever the mouse is moved.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp. Returning +1+ will not stop the mouse movement,
  #   but it will block the mouse notifications being sent to SketchUp. This
  #   means +onMouseMove+, +onMouseEnter+, and +onMouseLeave+ callback functions
  #   of +Sketchup::Tool+ will not be called.
  def hk_onMouseMove(x,y)
  end


  # Called when the left mouse button is pressed.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onLButtonDown(x,y)
  end


  # Called when the left mouse button is released.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onLButtonUp(x,y)
  end


  # Called when the left mouse button is double clicked.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onLButtonDoubleClick(x,y)
  end


  # Called when the right mouse button is pressed.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onRButtonDown(x,y)
  end


  # Called when the right mouse button is released.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onRButtonUp(x,y)
  end


  # Called when the right mouse button is double clicked.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onRButtonDoubleClick(x,y)
  end


  # Called when the middle mouse button is pressed.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onMButtonDown(x,y)
  end


  # Called when the middle mouse button is released.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onMButtonUp(x,y)
  end


  # Called when the middle mouse button is double clicked.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onMButtonDoubleClick(x,y)
  end


  # Called when the X mouse button is pressed.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onXButtonDown(x,y)
  end


  # Called when the X mouse button is released.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onXButtonUp(x,y)
  end


  # Called when the X mouse button is double clicked.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onXButtonDoubleClick(x,y)
  end


  # Called when the mouse wheel is rotated.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @param [Integer] dir Rotate direction:
  #   **+1+** - up (away from user)
  #   **+-1+** - down (toward the user)
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp. Blocking this message will stop the zoom in/out
  #   with the mouse wheel. This is very useful when the user wants to create
  #   an FPS type game that will use mouse wheel for an alternative operation,
  #   such as changing weapons.
  def hk_onMouseWheelRotate(x,y, dir)
  end


  # Called when the mouse wheel is tilted. Not all mouses are supported with
  # this feature. Be always ready to have multiple controls for the same
  # operation.
  # @param [Integer] x X position
  # @param [Integer] y Y position
  # @param [Integer] dir Tilt direction: +-1+ - left, +1+ - right.
  # @return [Integer] May return +1+ to prevent system from passing the message
  #   to SketchUp window procedure. Any other return value will let the message
  #   interfere with SketchUp.
  def hk_onMouseWheelTilt(x,y, dir)
  end

end
