# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

dir = File.dirname(__FILE__)
require File.join(dir, 'core.rb')

AMS::Lib.require 'windows.rb'

# A wrap-up utility for Win32::API functions.
module AMS::Window

  class << self

    private

    def win
      AMS::Windows
    end

    public

    # Determines whether the specified window handle identifies an existing
    # window.
    # @param [Long] hwnd A handle to the window to be tested.
    # @return [Boolean]
    def valid?(hwnd)
      win::IsWindow.call(hwnd) == 1
    end


    # Determines whether the specified window handle is the window with which the
    # user is currently working.
    # @param [Long] hwnd A handle to the window to be tested.
    # @return [Boolean]
    def foreground?(hwnd)
      win::GetForegroundWindow.call() == hwnd
    end

    alias active? foreground?


    # Determines whether the specified window handle has a menubar.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean]
    def has_menubar?(hwnd)
      menu = win::GetMenu.call(hwnd)
      menu != 0
    end


    # Determines whether a window is a child window or descendant window of a
    # specified parent window.
    # @param [Long] parent A handle to the parent window.
    # @param [Long] hwnd A handle to the window to be tested.
    # @ return [Boolean]
    def child?(parent, hwnd)
      win::IsChild.call(parent, hwnd) == 1
    end


    # Determines visibility state of the specified window.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean]
    def visible?(hwnd)
      win::IsWindowVisible.call(hwnd) == 1
    end


    # Determines whether the specified window is maximized.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean]
    def maximized?(hwnd)
      win::IsZoomed.call(hwnd) == 1
    end


    # Determines whether the specified window is minimized.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean]
    def minimized?(hwnd)
      win::IsIconic.call(hwnd) == 1
    end


    # Determines whether the specified window is restored (not maximized, nor
    # minimized).
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean]
    def restored?(hwnd)
      return false unless valid?(hwnd)
      !(maximized?(hwnd) or minimized?(hwnd))
    end


    # Retrieves a handle to the desktop window. The desktop window covers the
    # entire screen. The desktop window is the area on top of which other windows
    # are painted.
    # @return [Long] A handle to the window.
    def get_desktop
      win::GetDesktopWindow.call()
    end


    # Retrieves a handle to the foreground window (the window with which the user
    # is currently working).
    # @return [Long] A handle to the window.
    def get_foreground
      win::GetForegroundWindow.call()
    end

    alias get_active get_foreground


    # Retrieves class name of the specified window.
    # @param [Long] hwnd A handle to the window.
    # @return [String, nil] Window's class name.
    def class_name(hwnd)
      return nil unless valid?(hwnd)
      cname = 0.chr*64
      win::GetClassName.call(hwnd, cname, cname.size)
      cname.strip
    end


    # Retrieves an identifier to the process that created the specified window.
    # @param [Long] hwnd A handle to the window.
    # @return [Long] process id
    def process_id(hwnd)
      struct = 0.chr*4
      win::GetWindowThreadProcessId.call(hwnd, struct)
      struct.unpack('l')[0]
    end

    alias pid process_id


    # Retrieves an identifier to the thread that created the specified window.
    # @param [Long] hwnd A handle to the window.
    # @return [Long] thread id
    def thread_id(hwnd)
      win::GetWindowThreadProcessId.call(hwnd, nil)
    end

    alias tid thread_id


    # Retrieves a handle to the specified window's parent or owner.
    # @param [Long] hwnd A handle to the window.
    # @return [Long, nil] If the window is a child window, the return value is a
    #   handle to the parent window.
    def get_parent(hwnd)
      parent = win::GetParent.call(hwnd)
      return nil if parent == 0
      parent
    end


    # Retrieves title text of the specified window.
    # @param [Long] hwnd A handle to the window.
    # @return [String, nil] caption
    def get_caption(hwnd)
      return nil unless valid?(hwnd)
      size = win::GetWindowTextLength.call(hwnd)
      lpStr = 0.chr*size
      win::GetWindowText.call(hwnd, lpStr, (lpStr.size+1))
      lpStr
    end


    # Retrieves a handle to menubar of the specified window.
    # @param [Long] hwnd A handle to the window.
    # @return [Long, nil] hMenu
    def get_menubar(hwnd)
      return nil unless valid?(hwnd)
      menu = win::GetMenu.call(hwnd)
      return nil if menu == 0
      menu
    end


    # Retrieves dimensions (upper-left and lower-right corners) of the window's
    # bounding rectangle.
    # @param [Long] hwnd A handle to the window.
    # @param [Integer] mode Origin or frame of reference:
    #   +1+ : The dimensions are given in screen coordinates that are relative
    #   to the upper-left corner of the screen.
    #   +2+ : The dimensions are given in window coordinates that are relative
    #   to the upper-left corner of the window; the coordinates of the upper-left
    #   corner are (0,0).
    #   +3+ : The dimensions are given in client coordinates that are relative
    #   to the upper-left corner of the window's client area.
    # @return [Array, nil] +[x1,y1, x2,y2]+
    def rect(hwnd, mode = 1)
      return nil unless valid?(hwnd)
      mode = mode.to_i
      mode = 1 unless mode.between?(1,3)
      lpRect = 0.chr*16
      win::GetWindowRect.call(hwnd, lpRect)
      rect = lpRect.unpack('l*')
      if mode == 2
        rect[2] -= rect[0]
        rect[3] -= rect[1]
        rect[0..1] = [0,0]
      elsif mode == 3
        win::GetClientRect.call(hwnd, lpRect)
        style = win::GetWindowLong.call(hwnd, -16)
        exstyle = win::GetWindowLong.call(hwnd, -20)
        win::AdjustWindowRectEx.call(lpRect, style, has_menubar?(hwnd), exstyle)
        rect = lpRect.unpack('l*')
      end
      rect
    end


    # Retrieves coordinates of the upper-left corner of the window.
    # @param [Long] hwnd A handle to the window.
    # @param [Integer] mode Origin or frame of reference:
    #   +1+ : The dimensions are given in screen coordinates that are relative
    #   to the upper-left corner of the screen.
    #   +2+ : The dimensions are given in client coordinates that are relative
    #   to the upper-left corner of the window's client area.
    # @return [Array, nil] +[x,y]+
    def origin(hwnd, mode = 1)
      return nil unless valid?(hwnd)
      mode = mode.to_i
      mode = 1 unless mode.between?(1,2)
      mode = 3 if mode == 2
      rect(hwnd, mode)[0..1]
    end


    # Retrieves window's size in pixels.
    # @param [Long] hwnd A handle to the window.
    # @return [Array, nil] +[width,height]+
    def size(hwnd)
      return nil unless valid?(hwnd)
      rect(hwnd, 2)[2..3]
    end


    # Retrieves window's width in pixels.
    # @param [Long] hwnd A handle to the window.
    # @return [Integer, nil] width
    def width(hwnd)
      return nil unless valid?(hwnd)
      rect(hwnd, 2)[2]
    end


    # Retrieves window's height in pixels.
    # @param [Long] hwnd A handle to the window.
    # @return [Integer, nil] height
    def height(hwnd)
      return nil unless valid?(hwnd)
      rect(hwnd, 2)[3]
    end


    # Retrieves dimensions (upper-left and lower-right corners) of the window's
    # client area.
    # @param [Long] hwnd A handle to the window.
    # @param [Integer] mode Origin or frame of reference:
    #   +1+ : The dimensions are given in screen coordinates that are relative
    #   to the upper-left corner of the screen.
    #   +2+ : The dimensions are given in window coordinates that are relative
    #   to the upper-left corner of the window.
    #   +3+ : The dimensions are given in client coordinates that are relative
    #   to the upper-left corner of the window's client area; the coordinates of
    #   the upper-left corner are (0,0).
    # @return [Array, nil] +[x1,y1, x2,y2]+
    def client_rect(hwnd, mode = 1)
      return nil unless valid?(hwnd)
      mode = mode.to_i
      mode = 1 unless mode.between?(1,3)
      lpRect = 0.chr*16
      win::GetClientRect.call(hwnd, lpRect)
      crect = lpRect.unpack('l*')
      return crect if mode == 3
      style = win::GetWindowLong.call(hwnd, -16)
      exstyle = win::GetWindowLong.call(hwnd, -20)
      win::AdjustWindowRectEx.call(lpRect, style, has_menubar?(hwnd), exstyle)
      arect = lpRect.unpack('l*')
      r = []
      if mode == 1
        win::GetWindowRect.call(hwnd, lpRect)
        rect = lpRect.unpack('l*')
        r[0] = rect[0] - arect[0]
        r[1] = rect[1] - arect[1]
        r[2] = r[0] + crect[2]
        r[3] = r[1] + crect[3]
      else
        r[0] = -arect[0]
        r[1] = -arect[1]
        r[2] = r[0] + crect[2]
        r[3] = r[1] + crect[3]
      end
      r
    end


    # Retrieves coordinates of the upper-left corner of the window's client area.
    # @param [Long] hwnd A handle to the window.
    # @param [Integer] mode Origin or frame of reference:
    #   +1+ : The dimensions are given in screen coordinates that are relative
    #   to the upper-left corner of the screen.
    #   +2+ : The dimensions are given in window coordinates that are relative
    #   to the upper-left corner of the window.
    # @return [Array, nil] +[x,y]+
    def client_origin(hwnd, mode = 1)
      return nil unless valid?(hwnd)
      mode = mode.to_i
      mode = 1 unless mode.between?(1,2)
      client_rect(hwnd, mode)[0..1]
    end


    # Retrieves size of the window's client area in pixels.
    # @param [Long] hwnd A handle to the window.
    # @return [Array, nil] +[width,height]+
    def client_size(hwnd)
      return nil unless valid?(hwnd)
      client_rect(hwnd, 3)[2..3]
    end


    # Retrieves window's client width in pixels.
    # @param [Long] hwnd A handle to the window.
    # @return [Integer, nil] width
    def client_width(hwnd)
      return nil unless valid?(hwnd)
      client_rect(hwnd, 3)[2]
    end


    # Retrieves window's client height in pixels.
    # @param [Long] hwnd A handle to the window.
    # @return [Integer, nil] height
    def client_height(hwnd)
      return nil unless valid?(hwnd)
      client_rect(hwnd, 3)[3]
    end


    # Brings the thread that created the specified window into the foreground and
    # activates the window.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean] +true+ if the window was brought to the foreground.
    def set_foreground(hwnd)
      win::SetForegroundWindow.call(hwnd) == 1
    end

    alias set_active set_foreground


    # Changes the parent window of the specified child window.
    # @param [Long] hwnd A handle to the window.
    # @param [Long, nil] parent A handle to the new parent window. If this
    #   parameter is nil, the desktop window becomes the new parent window.
    # @ return [Boolean] +true+ if the function succeeds.
    def set_parent(hwnd, parent)
      win::SetParent.call(hwnd, parent) != 0
    end


    # Modifies window's title text.
    # @param [Long] hwnd A handle to the window.
    # @param [String] text New caption.
    # @return [Boolean] Whether the text was changed.
    def set_caption(hwnd, text)
      return false unless valid?(hwnd)
      text = text.to_s
      return false if text == get_caption(hwnd)
      win::SetWindowText.call(hwnd, text)
      true
    end


    # Assigns a new menu to the specified window.
    # @param [Long] hwnd A handle to the window.
    # @param [Long, nil] hmenu A handle to the menubar, or +nil+ to remove menubar.
    # @return [Boolean] +true+ if the function succeeds.
    def set_menubar(hwnd, hmenu)
      win::SetMenu.call(hwnd, hmenu) == 1
    end


    # Removes menubar from the specified window.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean] +true+ if the function succeeds.
    def remove_menubar(hwnd)
      set_menubar(hwnd, nil)
    end


    # Modifies visibility state of the specified window.
    # @param [Long] hwnd A handle to the window.
    # @param [Boolean] state Whether to show or hide the window.
    # @return [Boolean] Whether window's visibility state was changed.
    def show(hwnd, state = true)
      state = (state == true)
      return false if visible?(hwnd) == state
      cmd = state ? 5 : 0
      win::ShowWindow.call(hwnd, cmd)
      true
    end


    # Sets the specified window maximized.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean] Whether window's placement state was changed.
    def maximize(hwnd)
      return false if maximized?(hwnd)
      win::ShowWindow.call(hwnd, 3)
      true
    end


    # Sets the specified window minimized.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean] Whether window's placement state was changed.
    def minimize(hwnd)
      return false if minimized?(hwnd)
      win::ShowWindow.call(hwnd, 2)
      true
    end


    # Sets the specified window to default placement.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean] Whether window's placement state was changed.
    def restore(hwnd)
      return false if restored?(hwnd)
      win::ShowWindow.call(hwnd, 1)
      true
    end


    # Changes the position and dimensions of the specified window. For a top-level
    # window, the position and dimensions are relative to the upper-left corner of
    # the screen. For a child window, they are relative to the upper-left corner
    # of the parent window's client area.
    # @param [Long] hwnd A handle to the window.
    # @param [Integer] x
    # @param [Integer] y
    # @param [Integer] width
    # @param [Integer] height
    # @return [Boolean] +true+ if the function succeeds.
    def move(hwnd, x, y, width, height)
      win::MoveWindow.call(hwnd, x, y, width, height, true) == 1
    end


    # Repositions all dialogs and child windows of the specified window.
    # @note This method calls <i>WM_SIZE</i> message, which is not the most
    #   appropriate way to refresh the window. Improvements to this method are
    #   attempted!
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean] +true+ if the function succeeds.
    def refresh(hwnd)
      return false unless valid?(hwnd)
      # WM_SIZE (0x0005):
      #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632646(v=vs.85).aspx
      # See here how to set high-order word and low-order word:
      #   http://www.quickmacros.com/help/Other/IDP_HIWORD.html
      win::PostMessage.call(hwnd, 0x0005, nil, nil) == 1
    end


    # Sent as a signal that a window or an application should terminate. This
    # function does the same operation as clicking the close button.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean] +true+ if the function succeeds.
    def close(hwnd)
      return false unless valid?(hwnd)
      win::SendMessage.call(hwnd, 0x0010, nil, nil) == 1
    end


    # Forcibly closes the specified window.
    # @param [Long] hwnd A handle to the window.
    # @return [Boolean] +true+ if the function succeeds.
    def end_task(hwnd)
      win::EndTask.call(hwnd, false, true) == 1
    end

  end # proxy class

end # module AMS::Window

AMS::Lib.file_loaded(__FILE__)
